import { useNavigate, useParams } from "react-router-dom";
import MyModal, {
  MyModalCloseCreateButtonFooter,
  MyModalCloseSaveButtonFooter,
} from "../../Components/MyModal";
import { AppStyle, Constants, myFetch, myFetchContentType } from "../../utils";
import { Button, Card, Col, Row, Select, Typography } from "antd";
import { createRef, useRef, useState } from "react";
import TextArea from "antd/es/input/TextArea";
import Webcam from "react-webcam";
import {
  CameraOutlined,
  DeleteOutlined,
  FullscreenOutlined,
  PlusOutlined,
} from "@ant-design/icons";
import { DocumentationImage } from ".";

function UploadComponent({ index, setImagePreview }) {
  const handleFileChange = (event) => {
    const file = event.target.files[0];
    console.log("file", event.target.files);
    if (file) {
      const reader = new FileReader();
      reader.onload = () => {
        //setImagePreview(URL.createObjectURL(file));
        setImagePreview(reader.result);

        // base 64 string
        // console.log(reader.result);
      };
      reader.readAsDataURL(file);
    }
  };

  return (
    <div>
      <input
        type="file"
        accept="image/*"
        id={`upload-input-${index}`}
        style={{ display: "none" }}
        onChange={handleFileChange}
      />
      <label
        htmlFor={`upload-input-${index}`}
        style={{ cursor: "pointer", display: "inline-block" }}
      >
        <PlusOutlined />
      </label>
    </div>
  );
}

function CameraComponent({ setImagePreview }) {
  const [modalVisible, setModalVisible] = useState(false);
  const [cameraVisible, setCameraVisible] = useState(false);
  const webcamRef = useRef(null);

  const handleCapture = () => {
    const imageSrc = webcamRef.current.getScreenshot();
    setImagePreview(imageSrc);

    handleCancel();
  };

  const handleCancel = () => {
    setCameraVisible(false);
    // this is needed to close the camera correctly
    setTimeout(() => setModalVisible(false), 100);
  };

  return (
    <>
      <div
        onClick={() => {
          setModalVisible(true);
          setCameraVisible(true);
        }}
      >
        <CameraOutlined />
      </div>

      <MyModal
        isOpen={modalVisible}
        onCancel={handleCancel}
        footer={[
          <Button key={0} block type="primary" onClick={handleCapture}>
            Take picture
          </Button>,
        ]}
      >
        {cameraVisible && (
          <Webcam
            audio={false}
            ref={webcamRef}
            screenshotFormat="image/jpeg"
            width={"100%"}
            videoConstraints={{
              width: 1920,
              height: 1080,
              //facingMode: { exact: "environment" },
            }}
          />
        )}
      </MyModal>
    </>
  );
}

export function NoteComponent({
  viewMode,
  index,
  image,
  onImageChange,
  description,
  onDescriptionChange,
  onDeleteImage,
}) {
  return (
    <Row
      gutter={AppStyle.grid.row.glutter}
      style={{ marginBottom: AppStyle.app.marginBottom }}
    >
      <Col xs={24} md={8}>
        <Card
          bodyStyle={{ padding: 0 }}
          actions={
            viewMode
              ? [<FullscreenOutlined />]
              : [
                  <UploadComponent
                    index={index}
                    setImagePreview={onImageChange}
                  />,
                  <CameraComponent setImagePreview={onImageChange} />,
                  <DeleteOutlined onClick={onDeleteImage} />,
                  <FullscreenOutlined />,
                ]
          }
        >
          <DocumentationImage
            image={image}
            imgStyle={{ borderTopLeftRadius: 6, borderTopRightRadius: 6 }}
          />
        </Card>
      </Col>

      <Col xs={24} md={16}>
        {viewMode ? (
          <Typography.Text>{description}</Typography.Text>
        ) : (
          <TextArea
            rows={8}
            placeholder="Description"
            value={description}
            onChange={onDescriptionChange}
          />
        )}
      </Col>
    </Row>
  );
}

const emptyNote = { image: null, description: "" };

const selectDocumentationTypeOptions = [
  { value: 0, label: "Repair protocol" },
  { value: 1, label: "Documentation" },
];

export default function CreateEquipmentDocumentationModal({ isOpen }) {
  const navigate = useNavigate();
  let { paramStockItemId } = useParams();

  const [title, setTitle] = useState("New documentation");
  const [selectedDocumentationType, setSelectedDocumentationType] = useState(
    selectDocumentationTypeOptions[0].value
  );
  const [notes, setNotes] = useState([emptyNote]);

  const handleCancel = () =>
    navigate(Constants.ROUTE_PATHS.EQUIPMENT_DOCUMENTATION);

  const handleCreate = () => {
    let obj = {
      stockItemId: paramStockItemId,
      type: selectedDocumentationType,
      title: title,
      notes: notes,
    };

    myFetch(`/equipment/documentation/create`, "POST", obj, {}).then((data) => {
      console.log("data", data);
    });
  };

  const handleDescriptionChange = (index) => (e) => {
    const updatedNotes = [...notes];

    updatedNotes[index] = {
      ...updatedNotes[index],
      description: e.target.value,
    };

    setNotes(updatedNotes);
  };

  const handleImageChange = (index) => (newImage) => {
    const updatedNotes = [...notes];

    updatedNotes[index] = {
      ...updatedNotes[index],
      image: newImage,
    };

    setNotes(updatedNotes);
  };

  const handleAddNote = () => setNotes([...notes, emptyNote]);

  const isAddNoteButtonDisabled = () => {
    const lastNote = notes[notes.length - 1];

    return lastNote.image === null && lastNote.description === "";
  };

  return (
    <MyModal
      isOpen={isOpen}
      onCancel={handleCancel}
      footer={
        <MyModalCloseCreateButtonFooter
          onCreate={handleCreate}
          onCancel={handleCancel}
        />
      }
    >
      <Typography.Title
        editable={{ text: title, onChange: setTitle }}
        level={1}
      >
        {title}
      </Typography.Title>

      <div style={{ marginBottom: AppStyle.typography.text.marginBottom }}>
        <Typography.Text>Documentation type</Typography.Text>
      </div>

      <Select
        defaultValue={selectedDocumentationType}
        style={{ width: "100%", marginBottom: AppStyle.app.marginBottom }}
        onChange={(value) => setSelectedDocumentationType(value)}
        options={selectDocumentationTypeOptions}
      />

      {notes.map((note, index) => (
        <NoteComponent
          key={index}
          index={index}
          image={note.image}
          onImageChange={handleImageChange(index)}
          description={note.description}
          onDescriptionChange={handleDescriptionChange(index)}
          onDeleteImage={() => handleImageChange(index)(null)}
        />
      ))}

      <div style={{ textAlign: "center" }}>
        <Button
          type="primary"
          disabled={isAddNoteButtonDisabled()}
          icon={<PlusOutlined />}
          onClick={handleAddNote}
        >
          Add note
        </Button>
      </div>
    </MyModal>
  );
}
